package wmo

import (
	"context"
	"gogs.mikescher.com/BlackForestBytes/goext/exerr"
)

func (c *Coll[TData]) decodeSingle(ctx context.Context, dec Decodable) (TData, error) {
	if c.customDecoder != nil {

		res, err := (*c.customDecoder)(ctx, dec)
		if err != nil {
			return *new(TData), exerr.Wrap(err, "failed to decode single entity with custom-decoder").Type("decoder", *c.customDecoder).Build()
		}
		return res, nil

	} else {

		var res TData

		err := dec.Decode(&res)
		if err != nil {
			return *new(TData), exerr.Wrap(err, "failed to decode single entity").Type("target-type", res).Build()
		}

		return res, nil

	}
}

func (c *Coll[TData]) decodeAll(ctx context.Context, cursor Cursorable) ([]TData, error) {
	if c.customDecoder != nil {

		res := make([]TData, 0, cursor.RemainingBatchLength())

		for cursor.Next(ctx) {
			entry, err := (*c.customDecoder)(ctx, cursor)
			if err != nil {
				return nil, exerr.Wrap(err, "failed to decode entity with custom-decoder").Type("decoder", *c.customDecoder).Build()
			}
			res = append(res, entry)
		}

		return res, nil

	} else {

		res := make([]TData, 0, cursor.RemainingBatchLength())

		err := cursor.All(ctx, &res)
		if err != nil {
			return nil, exerr.Wrap(err, "failed to batch-decode entity").Type("target-type", res).Build()
		}

		return res, nil

	}

}
